#--
# Copyright (c) 2010-2012 RightScale Inc
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#++

require 'rubygems'
require 'sqlite3'

Given /A set of RightScripts for MySQL promote operations\.$/ do
  st = ServerTemplate.find(@servers[1].server_template_href)
  @scripts_to_run = {}
  @scripts_to_run['restore'] = st.executables.detect { |ex| ex.name =~  /restore and become/i }
  @scripts_to_run['slave_init'] = st.executables.detect { |ex| ex.name =~ /slave init v2/ }
  @scripts_to_run['promote'] = st.executables.detect { |ex| ex.name =~ /promote to master/ }
  @scripts_to_run['backup'] = st.executables.detect { |ex| ex.name =~ /EBS backup/ }
  @scripts_to_run['terminate'] = st.executables.detect { |ex| ex.name =~ /TERMINATE/ }
# hardwired script! hax! (this is an 'anyscript' that users typically use to setup the master dns)
  @scripts_to_run['master_init'] = RightScript.new('href' => "/api/acct/2901/right_scripts/195053")
  @scripts_to_run['create_stripe'] = RightScript.new('href' => "/api/acct/2901/right_scripts/198381")
  @scripts_to_run['create_mysql_ebs_stripe'] = RightScript.new('href' => "/api/acct/2901/right_scripts/212492")
end

Then /^I should run a mysql query "([^\"]*)" on server "([^\"]*)"\.$/ do |query, server_index|
  human_index = server_index.to_i - 1
  query_command = "echo -e \"#{query}\"| mysql"
  @servers[human_index].spot_check(query_command) { |result| puts result }
end

Then /^I should setup admin and replication privileges on server "([^\"]*)"\.$/ do |server_index|
  human_index = server_index.to_i - 1
  admin_grant = "grant all privileges on *.* to admin_user@\'%\' identified by \'admin_passwd\'"
  rep_grant = "grant replication slave on *.* to replication_user@\'%\' identified by \'replication_passwd\'"
  admin_grant2 = "grant all privileges on *.* to admin_user@\'localhost\' identified by \'admin_passwd\'"
  rep_grant2 = "grant replication slave on *.* to replication_user@\'localhost\' identified by \'replication_passwd\'"
  [admin_grant, rep_grant, admin_grant2, rep_grant2].each do |query|
    query_command = "echo -e \"#{query}\"| mysql"
    puts @servers[human_index].spot_check_command(query_command)
  end
end

Then /I should set a variation lineage./ do
  @lineage = "text:testlineage#{rand(1000000)}"
  @deployment.set_input('db/backup/lineage', @lineage)
# unset all server level inputs in the deployment to ensure use of 
# the setting from the deployment level
  @deployment.servers_no_reload.each do |s|
    s.set_input('db/backup/lineage', "text:")
  end
end

Then /I should set a variation backup prefix./ do
  @lineage = "text:testlineage#{rand(1000000)}"
  @deployment.set_input('DB_EBS_PREFIX', @lineage)
# unset all server level inputs in the deployment to ensure use of 
# the setting from the deployment level
  @deployment.servers_no_reload.each do |s|
    s.set_input('DB_EBS_PREFIX', "text:")
  end
end

Then /I should set an oldschool variation lineage./ do
  @lineage = "text:testlineage#{rand(1000000)}"
  @deployment.set_input('DB_LINEAGE_NAME', @lineage)
# unset all server level inputs in the deployment to ensure use of 
# the setting from the deployment level
  @deployment.servers_no_reload.each do |s|
    s.set_input('DB_LINEAGE_NAME', "text:")
  end
  puts "Using Lineage: #{@lineage}"
end

Then /^I should create a MySQL EBS stripe on server "([^\"]*)"\.$/ do |server_index|
  human_index = server_index.to_i - 1
# this needs to match the deployments inputs for lineage and stripe count.
  options = { "EBS_MOUNT_POINT" => "text:/mnt/mysql", 
              "EBS_STRIPE_COUNT" => "text:#{@stripe_count}", 
              "EBS_VOLUME_SIZE_GB" => "text:1", 
              "DBAPPLICATION_USER" => "text:someuser", 
              "DBAPPLICATION_PASSWORD" => "text:somepass", 
              "EBS_VOLUME_SIZE_GB" => "text:1", 
              "EBS_LINEAGE" => @lineage }
  @status = @servers[human_index].run_executable(@scripts_to_run['create_mysql_ebs_stripe'], options)
end

Then /^I should create an EBS stripe on server "([^\"]*)"\.$/ do |server_index|
  human_index = server_index.to_i - 1
# this needs to match the deployments inputs for lineage and stripe count.
  options = { "EBS_MOUNT_POINT" => "text:/mnt/mysql", 
              "EBS_STRIPE_COUNT" => "text:#{@stripe_count}", 
              "EBS_VOLUME_SIZE_GB" => "text:1", 
              "EBS_LINEAGE" => @lineage }
  @status = @servers[human_index].run_executable(@scripts_to_run['create_stripe'], options)
end

Then /^I should set a variation stripe count of "([^\"]*)"\.$/ do |stripe|
  @stripe_count = stripe
  @deployment.set_input("EBS_STRIPE_COUNT", "text:#{@stripe_count}")
end

Then /^I should set a variation MySQL DNS\./ do
    mydb = File.join(File.dirname(__FILE__),"..","shared.db")
    @dns_result = nil
    rowid = nil
    success = nil
    SQLite3::Database.new(mydb) do |db|
      while(1) 
        db.query("SELECT rowid FROM mysql_dns WHERE owner IS NULL") do |result|
          rowid = result.entries.first
        end
        raise "FATAL: unable to find available DNS_IDs to use!" unless rowid
        handle = db.query("UPDATE OR IGNORE mysql_dns SET owner='#{@deployment.nickname}' WHERE rowid = '#{rowid}'")
        handle.close
        success = db.get_first_row("SELECT owner FROM mysql_dns WHERE rowid = '#{rowid}'").first == @deployment.nickname
        break if success
        puts "retrying for lock on DNSIDs"
      end
      db.query("SELECT MASTER_DB_DNSNAME,MASTER_DB_DNSID,SLAVE_DB_DNSNAME,SLAVE_DB_DNSID FROM mysql_dns WHERE owner = '#{@deployment.nickname}'") do |result|
        @dns_result = result.entries.first
      end
    end
    @deployment.set_input("MASTER_DB_DNSNAME", @dns_result[0])
    @deployment.set_input("MASTER_DB_DNSID", @dns_result[1])
    @deployment.set_input("SLAVE_DB_DNSNAME", @dns_result[2])
    @deployment.set_input("SLAVE_DB_DNSID", @dns_result[3])
end

Then /^I should release the dns records for use with other deployments\.$/ do
  require 'sqlite3'
  SQLite3::Database.new(File.join(File.dirname(__FILE__), "..", "shared.db")) do |db|
    q = db.query("UPDATE mysql_dns SET owner=NULL where owner LIKE '#{@deployment.nickname}%'")
    q.close
  end
end

Then /^I should setup master dns to point at server "([^\"]*)"\.$/ do |server_index|
  human_index = server_index.to_i - 1
  audit = @servers[human_index].run_executable(@scripts_to_run['master_init']) #, {'DB_TEST_MASTER_DNSID' => @dns_result[1]})
  audit.wait_for_completed
  #@servers[human_index].run_script(@scripts_to_run['master_init']) #, {'DB_TEST_MASTER_DNSID' => @dns_result[1]})
end

Then /I should stop the mysql servers\.$/ do
  if @scripts_to_run['terminate']
    @servers.each { |s| s.run_executable(@scripts_to_run['terminate']) unless s.state == 'stopped' }
  else
    @servers.each { |s| s.stop }
  end

  @servers.each { |s| s.wait_for_state("stopped") }
# unset dns in our local cached copy..
  @servers.each { |s| s.dns_name = "" }
end
